import 'package:flutter/material.dart';
import 'package:markdown_editable_textinput/format_markdown.dart';

class MarkdownTextInput extends StatefulWidget {
  final Function onTextChanged;
  final String initialValue;
  final Function validators;
  final String label;

  MarkdownTextInput(
    this.onTextChanged,
    this.initialValue, {
    this.label,
    this.validators,
  });

  @override
  _MarkdownTextInputState createState() => _MarkdownTextInputState();
}

class _MarkdownTextInputState extends State<MarkdownTextInput> {
  final _controller = TextEditingController();

  void onTap(MarkdownType type, {int titleSize = 1}) {
    final basePosition = _controller.selection.baseOffset;

    final result = FormatMarkdown.convertToMarkdown(
        type, _controller.text, _controller.selection.baseOffset, _controller.selection.extentOffset,
        titleSize: titleSize);

    _controller.value = _controller.value
        .copyWith(text: result.data, selection: TextSelection.collapsed(offset: basePosition + result.cursorIndex));
  }

  @override
  void initState() {
    _controller.text = widget.initialValue;
    _controller.addListener(() {
      widget.onTextChanged(_controller.text);
    });
    super.initState();
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      decoration: BoxDecoration(
        color: Theme.of(context).cardColor,
        border: Border.all(color: Theme.of(context).accentColor, width: 2),
        borderRadius: const BorderRadius.all(Radius.circular(10)),
      ),
      child: Column(
        children: <Widget>[
          TextFormField(
            textInputAction: TextInputAction.newline,
            maxLines: null,
            controller: _controller,
            textCapitalization: TextCapitalization.sentences,
            validator: widget.validators != null ? (value) => widget.validators(value) as String : null,
            cursorColor: Theme.of(context).primaryColor,
            decoration: InputDecoration(
              enabledBorder: UnderlineInputBorder(borderSide: BorderSide(color: Theme.of(context).accentColor)),
              focusedBorder: UnderlineInputBorder(borderSide: BorderSide(color: Theme.of(context).accentColor)),
              hintText: widget.label,
              hintStyle: const TextStyle(color: Color.fromRGBO(63, 61, 86, 0.5)),
              contentPadding: const EdgeInsets.symmetric(vertical: 15, horizontal: 10),
            ),
          ),
          Material(
            color: Theme.of(context).cardColor,
            borderRadius: const BorderRadius.only(bottomLeft: Radius.circular(10), bottomRight: Radius.circular(10)),
            child: Row(
              children: [
                InkWell(
                  key: const Key('bold_button'),
                  onTap: () => onTap(MarkdownType.bold),
                  child: Padding(
                    padding: const EdgeInsets.all(10),
                    child: Icon(
                      Icons.format_bold,
                    ),
                  ),
                ),
                InkWell(
                  key: const Key('italic_button'),
                  onTap: () => onTap(MarkdownType.italic),
                  child: Padding(
                    padding: const EdgeInsets.all(10),
                    child: Icon(
                      Icons.format_italic,
                    ),
                  ),
                ),
                for (int i = 1; i <= 3; i++)
                  InkWell(
                    key: Key('H${i}_button'),
                    onTap: () => onTap(MarkdownType.title, titleSize: i),
                    child: Padding(
                      padding: const EdgeInsets.all(10),
                      child: Text(
                        'H$i',
                        style: TextStyle(fontSize: (18 - i).toDouble(), fontWeight: FontWeight.w700),
                      ),
                    ),
                  ),
                InkWell(
                  key: const Key('link_button'),
                  onTap: () => onTap(MarkdownType.link),
                  child: Padding(
                    padding: const EdgeInsets.all(10),
                    child: Icon(
                      Icons.link,
                    ),
                  ),
                ),
                InkWell(
                  key: const Key('list_button'),
                  onTap: () => onTap(MarkdownType.list),
                  child: Padding(
                    padding: const EdgeInsets.all(10),
                    child: Icon(
                      Icons.list,
                    ),
                  ),
                ),
              ],
            ),
          )
        ],
      ),
    );
  }
}
